from fastapi import Depends, APIRouter, HTTPException, status
from fastapi.responses import JSONResponse

from util.auth import check_auth
from model.dhcp_leases_model import DhcpLeaseDB
from model.wapi.cntl import Mgr, APIToken
from model.wapi.nd import BCD
from . import get_dhcp_lease_db, get_conn

router = APIRouter(
    prefix="/api/dhcp_leases",
    tags=["dhcp_leases"],
)
js_mods = ['dhcp_leases']


# TODO
@router.get("/bcds", deprecated=True)
def usr_bcds():
    raise DeprecationWarning('This endpoint was deprecated and now removed!')


@router.get("/bcds/{bcd_name}/active")
def active_leases(bcd_name: str,
                  conn=Depends(get_conn),
                  dhcp_lease_db: DhcpLeaseDB = Depends(get_dhcp_lease_db),
                  token: APIToken = Depends(check_auth)):
    user = Mgr.check_token(conn, token)
    bcd = BCD.get_by_name(conn, user.login_name, bcd_name)
    if bcd is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND)
    leases = dhcp_lease_db.get_active_macs(bcd_name)
    return JSONResponse(leases)
